{% extends "base.html" %}

{% block _navigation %}
    <nav id="main-nav">
        {% spaceless %}
            {% block navigation %}
                {% url workspace as current_path %} 
                {% if current_path == request.path %}
                    <span class="selected"><span class="icon workspace"></span> Workspace</span>
                {% else %}
                    <a href="{% url workspace %}"><span class="icon workspace"></span> Workspace</a>
                {% endif %}

                {% url define as current_path %} 
                {% if current_path == request.path %}
                    <span class="selected"><span class="icon query"></span> Query</span>
                {% else %}
                    <a href="{% url define %}"><span class="icon query"></span> Query</a>
                {% endif %}

                {% url report as current_path %} 
                {% if current_path == request.path %}
                    <span class="selected"><span class="icon databrowse"></span> Report</span>
                {% else %}
                    <a href="{% url report %}"><span class="icon databrowse"></span> Report</a>
                {% endif %}
            {% endblock %}
        {% endspaceless %}
    </nav>    
{% endblock %}


