define(["cilantro/rest/basext"],function(a){var b=a.extend({}),c=b.extend({_cache:undefined,get:function(a,b){b=b||!1,a=a||{};if(!b&&this._cache)return this._cache;var c=this,d=$.extend({},this.ajax,a),e=d.success;return d.success=function(a){var b=c.decode(a);c._cache=b,e(a,b)},this.xhr=$.ajax(d),this._cache}},{defargs:{ajax:{url:window.location,data:{},success:function(){},error:function(){},cache:!1},decode:function(a){return a}}});return{ajax:c}})