define(["cilantro/rest/basext"],function(a){var b=a.extend({},{defargs:{target:null}}),c=b.extend({render:function(a,b){b=b?b:this.replace,b===!0&&this.target.html("");if(this.bindData===!0){l=[];for(var c,d,e=0;e<a.length;e++)c=a[e],d=$.jqoteobj(this.template,c),d.data(c),l.push(d)}else l=[$.jqoteobj(this.template,a)];var f=this.target;return b==="prepend"?$.each(l.reverse(),function(){f.prepend(this)}):$.each(l,function(){f.append(this)}),this.target}},{defargs:{template:null,replace:!0,bindData:!1}});return{base:b,template:c}})