define(['cilantro/types/report/models', 'cilantro/types/report/views', 'cilantro/types/report/messages'], function(Models, Views, Messages) {
  return {
    Models: Models,
    Views: Views,
    Messages: Messages
  };
});