var __bind=function(a,b){return function(){return a.apply(b,arguments)}},__hasProp=Object.prototype.hasOwnProperty,__extends=function(a,b){function d(){this.constructor=a}for(var c in b)__hasProp.call(b,c)&&(a[c]=b[c]);return d.prototype=b.prototype,a.prototype=new d,a.__super__=b.prototype,a};define(["common/models/polling"],function(a){var b;return b=function(){function b(){this.revert=__bind(this.revert,this),b.__super__.constructor.apply(this,arguments)}return __extends(b,a.Model),b.prototype.url=App.endpoints.session.scope,b.prototype.initialize=function(){return b.__super__.initialize.apply(this,arguments),App.hub.subscribe("report/revert",this.revert)},b.prototype.revert=function(){return this.fetch()},b}(),{Session:b}})