var __hasProp=Object.prototype.hasOwnProperty,__extends=function(a,b){function d(){this.constructor=a}for(var c in b)__hasProp.call(b,c)&&(a[c]=b[c]);return d.prototype=b.prototype,a.prototype=new d,a.__super__=b.prototype,a};define(["underscore","common/views/state","cilantro/utils/search"],function(a,b,c){var d,e,f,g;return e=function(){function a(){a.__super__.constructor.apply(this,arguments)}return __extends(a,c.ResultCollection),a.prototype.url=App.endpoints.criteria,a}(),f=function(){function c(){c.__super__.constructor.apply(this,arguments)}return __extends(c,b.View),c.prototype.template=a.template('<span class="name"><%= name %></span>            <br><span class="info"><%= domain.name %></span>'),c.prototype.events={click:"click"},c.prototype.render=function(){return this.el.html(this.template(this.model.toJSON())),this},c.prototype.click=function(){return App.hub.publish("concept/request",this.model.id),!1},c}(),d=function(){function a(){a.__super__.constructor.apply(this,arguments)}return __extends(a,c.InputView),a.prototype.el="#concept-search",a}(),g=function(){function a(){a.__super__.constructor.apply(this,arguments)}return __extends(a,c.ResultListView),a.prototype.el="#concept-search-results",a.prototype.viewClass=f,a.prototype.inputViewClass=d,a}(),{ResultCollection:e,ResultListView:g}})