.column{display:-moz-inline-box;-moz-box-orient:vertical;display:inline-block;vertical-align:baseline;zoom:1;*display:inline;*vertical-align:auto;vertical-align:top;width:463px}.column:first-child{margin-right:10px}.column h2{margin-top:0}[role=modified]{display:none}.timestamp{margin:0;cursor:default;text-align:right}#new-report{-webkit-border-radius:3px;-moz-border-radius:3px;-ms-border-radius:3px;-o-border-radius:3px;border-radius:3px;background-color:#c3d871;background-image:-webkit-gradient(linear, left top, left bottom, color-stop(0%, #c3d871),color-stop(100%, #98b231));background-image:-webkit-linear-gradient(top, #c3d871,#98b231);background-image:-moz-linear-gradient(top, #c3d871,#98b231);background-image:-ms-linear-gradient(top, #c3d871,#98b231);background-image:-o-linear-gradient(top, #c3d871,#98b231);background-image:linear-gradient(top, #c3d871,#98b231);-webkit-box-shadow:0 1px 1px #a8ab9e;-moz-box-shadow:0 1px 1px #a8ab9e;-ms-box-shadow:0 1px 1px #a8ab9e;-o-box-shadow:0 1px 1px #a8ab9e;box-shadow:0 1px 1px #a8ab9e;border:1px solid #879e2b;border-bottom:1px solid #768a26;color:#333333;line-height:1;text-align:center;font-weight:bold;text-shadow:0 1px 1px #c3d871;font-size:1.3em;width:300px;margin-left:-150px;position:relative;left:50%}#new-report:hover{background-color:#b2cd49;background-image:-webkit-gradient(linear, left top, left bottom, color-stop(0%, #b2cd49),color-stop(100%, #768a26));background-image:-webkit-linear-gradient(top, #b2cd49,#768a26);background-image:-moz-linear-gradient(top, #b2cd49,#768a26);background-image:-ms-linear-gradient(top, #b2cd49,#768a26);background-image:-o-linear-gradient(top, #b2cd49,#768a26);background-image:linear-gradient(top, #b2cd49,#768a26);border:1px solid #768a26;border-bottom:1px solid #54621b;cursor:pointer;text-shadow:0 1px 1px #b2cd49}#new-report:active{-webkit-box-shadow:inset 0 0 5px #54621b,0 1px 2px #e0e6c9;-moz-box-shadow:inset 0 0 5px #54621b,0 1px 2px #e0e6c9;-ms-box-shadow:inset 0 0 5px #54621b,0 1px 2px #e0e6c9;-o-box-shadow:inset 0 0 5px #54621b,0 1px 2px #e0e6c9;box-shadow:inset 0 0 5px #54621b,0 1px 2px #e0e6c9;border:1px solid #54621b;border-bottom:1px solid #313a10}#new-report small{font-size:0.7em;color:#54621b}#session-report > div{zoom:1}#session-report > div:before,#session-report > div:after{content:"";display:table}#session-report > div:after{clear:both}#session-report > .column{display:-moz-inline-box;-moz-box-orient:vertical;display:inline-block;vertical-align:baseline;zoom:1;*display:inline;*vertical-align:auto;vertical-align:top;width:223px;margin-top:5px}#session-report > .column:first-child{margin-right:15px}#session-scope,#session-perspective{font-size:11px;list-style:disc;padding-left:30px;margin:0}
