var __hasProp=Object.prototype.hasOwnProperty,__extends=function(a,b){function d(){this.constructor=a}for(var c in b)__hasProp.call(b,c)&&(a[c]=b[c]);return d.prototype=b.prototype,a.prototype=new d,a.__super__=b.prototype,a};define(["underscore","backbone","common/models/state","common/views/state","common/views/collection"],function(a,b,c,d,e){var f,g,h,i,j;return f=function(){function a(){a.__super__.constructor.apply(this,arguments)}return __extends(a,c.Model),a}(),g=function(){function a(){a.__super__.constructor.apply(this,arguments)}return __extends(a,b.Collection),a.prototype.model=f,a.prototype.url=App.endpoints.session.scope,a.prototype.parse=function(a){return a.conditions},a}(),j=function(){function c(){c.__super__.constructor.apply(this,arguments)}return __extends(c,b.View),c.prototype.events={click:"click","click .remove":"remove"},c.prototype.template=a.template('<div class="criterion clearfix">                <a href="#" class="remove-criterion"></a>                <a href="#" class="field-anchor">                <%= condition %>                </a>            </div>'),c.prototype.render=function(){return this.el.remove(),this.el=this.$(this.template(this.model.toJSON())),this.delegateEvents(),this},c.prototype.click=function(){return this.model.activate(),!1},c.prototype.remove=function(){return this.model.remove(),!1},c}(),h=function(){function a(){a.__super__.constructor.apply(this,arguments)}return __extends(a,b.View),a.prototype.el="#condition-list-pane",a.prototype.events={"click #clear-conditions":"click"},a.prototype.initialize=function(a){return this.list=a.list},a.prototype.click=function(){return this.list.collection.remove(this.list.collection.models)},a}(),i=function(){function a(){a.__super__.constructor.apply(this,arguments)}return __extends(a,e.View),a.prototype.el="#condition-list",a.prototype.viewClass=j,a}(),{Collection:g,ListView:i,ListPane:h}})