{% extends "base_nav.html" %}

{% block title %}{{ block.super }} : Workspace{% endblock %}

{% block styles %}
    {{ block.super }}
    <link rel="stylesheet" href="{{ CILANTRO_CSS_URL }}workspace.css">
{% endblock %}

{% block scripts %}
    {{ block.super }}
    <script>require(['cilantro/pages/workspace/main']);</script>
{% endblock %}

{% block content %}

    <div class="container">
        <div class="title">Workspace</div>

        <div class="content">

            <div class="column">
                <h2>What You Were Working On Last</h2>

                <p class="info">This is a quick overview of your last session.</p> 

                <section class="report" id="session-report">
                    <div class="column">
                        <strong>Conditions</strong>
                        <ul id="session-scope"></ul>
                    </div>

                    <div class="column">
                        <strong>Columns</strong>
                        <ul id="session-perspective"></ul>
                    </div>
                </section>

                <hr>

                <button id="new-report">
                    New Report<br>
                    <small>(make sure to save any unsaved changes first)</small>
                </button>

            </div>


            <div class="column">
                <h2>Reports</h2>
                <p class="info">Lists all your saved reports.</p>
                <div id="report-list"></div>
            </div>

        </div>
    </div>

{% endblock %}
