define(["cilantro/vendor/base"],function(){var a=Base.extend({__defargslist:function(a,b){return a=a||this.constructor,b=b||[],a.ancestor&&(b=this.__defargslist(a.ancestor,b)),a.defargs&&typeof a.defargs=="object"&&b.push(a.defargs),b},_defargs:function(){var a=this.__defargslist();return $.extend.apply(this,[{}].concat(a))},constructor:function(a){var b=this._defargs(),c=$.extend({},b,a);for(var d in c)b.hasOwnProperty(d)&&(this[d]=c[d])}},{defargs:{}});return a})