var __hasProp=Object.prototype.hasOwnProperty,__extends=function(a,b){function d(){this.constructor=a}for(var c in b)__hasProp.call(b,c)&&(a[c]=b[c]);return d.prototype=b.prototype,a.prototype=new d,a.__super__=b.prototype,a},__bind=function(a,b){return function(){return a.apply(b,arguments)}};define(["backbone"],function(a){var b;return b=function(){function b(){b.__super__.constructor.apply(this,arguments)}return __extends(b,a.View),b.prototype.title="Learn about Reports",b.prototype.content='<div class="content">            <section>Hello World</section>            </div>',b.prototype.loadContent=null,b.prototype.initialize=function(a){this.loadContent?this.el.load(this.loadContent):this.el.html(this.content),this.el.dialog({autoOpen:!1,modal:!0,resizable:!1,draggable:!1,title:this.title});if(a.trigger)return this.$(a.trigger).click(__bind(function(){return this.el.dialog("open"),!1},this))},b}(),{Modal:b}})