var __bind=function(a,b){return function(){return a.apply(b,arguments)}},__hasProp=Object.prototype.hasOwnProperty,__extends=function(a,b){function d(){this.constructor=a}for(var c in b)__hasProp.call(b,c)&&(a[c]=b[c]);return d.prototype=b.prototype,a.prototype=new d,a.__super__=b.prototype,a};define(["backbone"],function(a){var b;return b=function(){function b(){this.disable=__bind(this.disable,this),this.enable=__bind(this.enable,this),this.inactivate=__bind(this.inactivate,this),this.activate=__bind(this.activate,this),b.__super__.constructor.apply(this,arguments)}return __extends(b,a.View),b.prototype.initialize=function(){return this.model.bind("active",this.activate),this.model.bind("inactive",this.inactivate),this.model.bind("enabled",this.enable),this.model.bind("disabled",this.disable)},b.prototype.activate=function(){return this.el.addClass("active")},b.prototype.inactivate=function(){return this.el.removeClass("active")},b.prototype.enable=function(){return this.el.show()},b.prototype.disable=function(){return this.el.hide()},b}(),{View:b}})