var __bind=function(a,b){return function(){return a.apply(b,arguments)}},__hasProp=Object.prototype.hasOwnProperty,__extends=function(a,b){function d(){this.constructor=a}for(var c in b)__hasProp.call(b,c)&&(a[c]=b[c]);return d.prototype=b.prototype,a.prototype=new d,a.__super__=b.prototype,a};define(["backbone","cilantro/define/criteriamanager","cilantro/define/conceptmanager"],function(a,b,c){var d;return d=function(){function d(){this.activate=__bind(this.activate,this),d.__super__.constructor.apply(this,arguments)}return __extends(d,a.View),d.prototype.el="#plugin-panel",d.prototype.initialize=function(){return App.hub.subscribe("concept/active",this.activate)},d.prototype.activate=function(a){var d;return d=b.retrieveCriteriaDS(a.id),c.isConceptLoaded(a.id)?c.show({id:a.id},d):a.fetch({beforeSend:__bind(function(){return this.el.block()},this),complete:__bind(function(){return this.el.unblock()},this),success:function(){return c.show(a.get("viewset"),d)}})},d}(),{ConceptInterfaceView:d}})