define({

    ACTIVATE_DESCRIPTION: 'activate-description',
    DEACTIVATE_DESCRIPTION: 'deactivate-description',

    ACTIVATE_CATEGORY: 'activate-category',
    SYNC_CATEGORY: 'sync-category',

    ACTIVATE_CRITERION: 'activate-criterion'

});
