var __bind=function(a,b){return function(){return a.apply(b,arguments)}},__hasProp=Object.prototype.hasOwnProperty,__extends=function(a,b){function d(){this.constructor=a}for(var c in b)__hasProp.call(b,c)&&(a[c]=b[c]);return d.prototype=b.prototype,a.prototype=new d,a.__super__=b.prototype,a};define(["backbone","common/utils"],function(a,b){var c,d,e;return c={pollInterval:1e4,initialize:function(){return this.startPolling()},startPolling:function(){return this._pollInterval=setInterval(__bind(function(){return this.poll()},this),this.pollInterval)},stopPolling:function(){return clearTimeout(this._pollInterval)}},e=function(){function b(){b.__super__.constructor.apply(this,arguments)}return __extends(b,a.Model),b.prototype.poll=function(){return this.fetch()},b}(),d=function(){function b(){b.__super__.constructor.apply(this,arguments)}return __extends(b,a.Collection),b.prototype.poll=function(){return this.fetch({update:!0})},b}(),b.include(e,c),b.include(d,c),{Model:e,Collection:d}})