var __bind=function(a,b){return function(){return a.apply(b,arguments)}},__hasProp=Object.prototype.hasOwnProperty,__extends=function(a,b){function d(){this.constructor=a}for(var c in b)__hasProp.call(b,c)&&(a[c]=b[c]);return d.prototype=b.prototype,a.prototype=new d,a.__super__=b.prototype,a};define(["underscore","common/views/collection","cilantro/types/report/main"],function(a,b,c){var d,e,f;return f=function(){function c(){this.render=__bind(this.render,this),c.__super__.constructor.apply(this,arguments)}return __extends(c,b.ExpandableList),c.prototype.el="#session-scope",c.prototype.defaultContent='<li class="info">No conditions have been defined</li>',c.prototype.initialize=function(){return this.model.bind("change:conditions",this.render),this.defaultContent=this.$(this.defaultContent),this.el.append(this.defaultContent)},c.prototype.render=function(){var b,c;return b=this.model.get("conditions"),this.defaultContent.detach(),this.el.empty(),b?(c="",a.map(b,function(a){return c+="<li>"+a.condition+"</li>"}),this.el.html(c)):this.el.append(this.defaultContent),this.collapse()},c}(),d=function(){function c(){this.render=__bind(this.render,this),c.__super__.constructor.apply(this,arguments)}return __extends(c,b.ExpandableList),c.prototype.el="#session-perspective",c.prototype.defaultContent='<li class="info">No data columns have been choosen</li>',c.prototype.initialize=function(){return this.model.bind("change:header",this.render),this.defaultContent=this.$(this.defaultContent),this.el.append(this.defaultContent)},c.prototype.template=a.template('<li><%= name %><% if (direction) { %>            <span class="info">(<%= direction %>)</span><% } %></li>'),c.prototype.render=function(){var a,b,c,d;this.defaultContent.detach(),this.el.empty();if(b=this.model.get("header"))for(c=0,d=b.length;c<d;c++)a=b[c],this.el.append(this.template(a));else this.el.append(this.defaultContent);return this.collapse()},c}(),e=function(){function a(){this.render=__bind(this.render,this),a.__super__.constructor.apply(this,arguments)}return __extends(a,c.Views.Item),a.prototype.el='<div>                <strong><a role="name"></a></strong>                <span class="info">- <span role="unique-count"></span> unique <span role="verbose-name"></span></span>                <span class="info time" style="float: right">modified <span role="modified"></span><span role="timesince"></span></span>                <div role="description"></div>            </div>',a.prototype.parent="#session-report",a.prototype.initialize=function(){return this.el.prependTo(this.parent),this.model.bind("change",this.render)},a.prototype.render=function(){var b;a.__super__.render.apply(this,arguments),b=this.model.get("name");if(this.model.hasChanged("name")&&b)return this.name.text(b).attr("href",this.model.get("permalink")).parent().show();if(!b)return this.name.text("(session)")},a}(),{Report:e,Scope:f,Perspective:d}})