require(["jquery","cilantro/types/report/main","cilantro/report/table","cilantro/report/columns"],function(a,b,c,d){var e;return e=new b.Models.Session,App.hub.subscribe("session/idle",function(){return e.stopPolling()}),App.hub.subscribe("session/resume",function(){return e.startPolling()}),a(function(){var f,g,h,i;return f=new b.Views.Editor,g=new b.Views.Name({model:e}),h=new b.Messages.UnsavedReport({model:e}),e.fetch(),d.init(),c.init(),i=a("#export-data"),i.bind("click",function(){return i.attr("disabled",!0),window.location=App.endpoints.session.report+"?data=1&format=csv",setTimeout(function(){return i.attr("disabled",!1)},5e3),!1})})})