var __slice=Array.prototype.slice,__bind=function(a,b){return function(){return a.apply(b,arguments)}};define(function(){var a,b,c,d;return b=function(a,b){var c,d,e;e=[];for(d in b)c=b[d],e.push(a[d]=c);return e},c=function(a,c){return b(a.prototype,c)},d=function(a,b,c){var d,e,f,g,h,i;arguments.length<3&&(h=[typeof exports!="undefined"?exports:window].concat(__slice.call(arguments)),a=h[0],b=h[1],c=h[2]),e=a,i=b.split(".");for(f=0,g=i.length;f<g;f++)d=i[f],a=a[d]||(a[d]={});return c(a,e)},a=function(){function a(a){a.pending||(a.pending=0),b(this,a)}return a.prototype._ready=function(){var a;if(this._queue)return;return this._queue=[],a=setInterval(__bind(function(){var b,c,d,e;if(this.pending!==0)return;this._isReady=!0,clearTimeout(a),e=this._queue;for(c=0,d=e.length;c<d;c++)b=e[c],b();return delete this._queue},this),25)},a.prototype.ready=function(a){return this._ready(),this._isReady?a():this._queue.push(a)},a}(),{extend:b,include:c,namespace:d,App:a}})